# frozen_string_literal: true

require_relative 'configuration'

module Minato
  module Utils
    class LogFormatter < ::Logger::Formatter
      def call(severity, timestamp, _program_name, data)
        json_str = {
          type:    severity,
          time:    timestamp,
          message: data.delete(:message)
        }.merge(data).to_json

        "#{json_str}\r\n"
      end
    end

    class BaseLogger
      def initialize(logger)
        @logger = logger
        @logger.formatter = LogFormatter.new if @logger
        @debug = Minato::Utils.config.debug
      end

      def log_request(method, host, path, options)
        return unless @logger

        call :info,
             message:      'Minato::Utils::Client starting request',
             http_request: request_data(method, host, path, options)
      end

      def log_response(method, host, path, options, response)
        return unless @logger

        call :info,
             message:       'Minato::Utils::Client receiving response',
             http_request:  request_data(method, host, path, options),
             http_response: response_data(response)
      end

      def call(type, message)
        @logger.send type, message
      end

      private

      def request_data(method, host, path, options)
        data = {
          method: method,
          host:   host,
          path:   path
        }

        data[:query] = options[:query] if options[:query]
        @debug ? data.merge({ headers: options[:headers] }) : data
      end

      def response_data(response)
        data = {
          code:    response.code,
          message: response.message
        }

        @debug ? data.merge({ headers: response.headers, body: response.body }) : data
      end
    end

    module_function

    def logger
      @logger ||= BaseLogger.new(Minato::Utils.config.logger.dup)
    end
  end
end
